
// IrModule.ino
// 
//       Updated by Andrey Zhunev, based on v.09 modified by David Frank 1-10-14.
//       This is the modified version of Randy Perkins' IrModule that includes camera control for the S107C
//          Changes compared to original v.09:
//            - Fixed periodic "unwanted" IR pulses when Throttle is at 0
//            - Improved robustness of IR channel selection (A/B)
//            - PPM channels sequence changed to RETA
//            - Aileron stick controls Camera
//                - Push stick to the right momoentarily to toggle video record
//                - Push stick to the left to take picture
//          This Modified version should work with both the S107G and S107C
// 
// GPL V2 license.  copyright 2013 Randy Perkins
// Derived work of IRrc library copyright  Mike McCauley http://www.airspayce.com/mikem/arduino/IRrc/index.html
// Derived work of "best way to read a servo signal on arduino" copyright DIY Drones (quix-fz) http://diydrones.com/profiles/blogs/705844:BlogPost:39393

// Control a Syma S107_R5 3 channel coaxial IR helicopter from a ppm_sum input
// sketch expects the JR TAER channel order on the PPM input
// sketch expects 8 channels of PPM data as some TX are hardwired to produce 8 channels
// sketch only the first 4 channels
// Uses arduino pin 8 for ppm_sum input ( Timer 1)
// Uses arduino pin 11 for IR output ( Timer 2)

// variables that will be different for each model as they are added
//  Protocol for Syma S107G reciever board R5 - the so called 32 bit version
//  Derived from http://www.rcgroups.com/forums/showpost.php?p=18118911&postcount=33
const int preamble = 2000;
const int preambleGap = 2000;
const int zeroOnTime = 360;
const int zeroOffTime = 240;
const int oneOnTime = 360;
const int oneOffTime = 640;
const int carrierFrequency = 38000;
const int irChannelSpacing [] = {
  120,180};// this is the channel dependant delay between data packets, reduces inteference between models 
const byte irDataStructure [] = {
  8,8,8,8}; // the S026 does not use a common data size in it's transmission, so we're ready

// global variables
int ppmInput[8]; //capture 8 channels of ppm data, throw away last 4
byte irChannel; // which IR channel should the data be sent out
byte irPicture;  // Take a picture ie camera shutter
byte irVideo;  // Record / stop record
const byte LED_OUTPUT_PIN = A0;  // this is a status led

// functions related to encoding IR data
void sendZero()
{
  TCCR2A |= _BV(COM2A0); // Enable pin 11 PWM output
  delayMicroseconds(zeroOnTime);
  TCCR2A &= ~(_BV(COM2A0)); // Disable pin 11 PWM output
  delayMicroseconds(zeroOffTime);
}
void sendOne()
{
  TCCR2A |= _BV(COM2A0); // Enable pin 11 PWM output
  delayMicroseconds(oneOnTime);
  TCCR2A &= ~(_BV(COM2A0)); // Disable pin 11 PWM output
  delayMicroseconds(oneOffTime);
}
void sendPreamble ()
{
  TCCR2A |= _BV(COM2A0); // Enable pin 11 PWM output
  delayMicroseconds(preamble);
  TCCR2A &= ~(_BV(COM2A0)); // Disable pin 11 PWM output
  delayMicroseconds(preambleGap);
}
void sendPostamble ()
{
  sendOne();
}
// function to toggle the status led
void toggleLed ()
{
  digitalWrite(LED_OUTPUT_PIN,!digitalRead(LED_OUTPUT_PIN));
}

// Interrupt routine used by PPM input decoding
ISR(TIMER1_CAPT_vect){
  static unsigned int lasticr; //icr at last capture
  static unsigned char cppmInput; //current input ppm channel
  unsigned int licr;
  licr=ICR1-lasticr;
  lasticr=ICR1;
  // remember .5us resolution.  this number is 2000us *2 plus 500 (250us) overtravel
  if(licr>4500){ //pulse too long, means start of new frame
    cppmInput=0;
  }
  // remember .5us resolution.  this number is 1000us *2 minus 500 (250us) undertravel
  else if(licr>1500){ //pulse good, take reading, go to next channel
    ppmInput[cppmInput]=licr;
    if(cppmInput<8){
      cppmInput++;
    }
  }
  else{
    //too short, nothing to see here
  }
}

void setup()
{
  // LED_OUTPUT_PIN is arduino pin A0
  // It is a global variable so defined earlier
  //  stasus led is wired active low on A0 because usbasp hardware is wired that way
  pinMode(LED_OUTPUT_PIN, OUTPUT);
  digitalWrite(LED_OUTPUT_PIN, HIGH);  // active low so HIGH is off

  // timer 1 for ppm decoding input
  // uses arduino pin 8 , but the pin doesnt need to be declared anywhere
  //const byte ppmInput_PIN = 8;// it has to be in pin 8 to use ICP
  //pinMode(ppmInput_PIN, INPUT);
  TIMSK1 &= ~( _BV(TOIE1) | _BV(ICIE1) | _BV(OCIE1A) | _BV(OCIE1B));
  TCCR1A &= ~( _BV(WGM11) | _BV(WGM10) );
  TCCR1B &= ~( _BV(WGM12) | _BV(WGM13) | _BV(ICNC1));
  //capture raising edge
  TCCR1B |= _BV(ICES1); //capture raising edge
  //prescaler 1/8
  TCCR1B |= _BV(CS11);
  TCCR1B &= ~( _BV(CS12) | _BV(CS10) );
  //disable outputs
  TCCR1A &= ~( _BV(COM1A0) | _BV(COM1A1) | _BV(COM1B0) | _BV(COM1B1));
  //enable capture interupt
  TIMSK1 |= _BV(ICIE1);

  // timer 2 for 38khz IR encoding output
  // hardware output compare pin must be set as an output
  const byte IR_OUTPUT_PIN = 11;
  pinMode(IR_OUTPUT_PIN, OUTPUT);
  TCCR2A = _BV(WGM21);
  TCCR2B = _BV(CS20);
  // hopefully this F_CPU will catch the usbasp at 12mhz properly
  OCR2A = F_CPU / 2 / carrierFrequency;

  // push aileron stick to the right to set alternate IR channel
  // pause 3 seconds to get the value
  while ( millis() < 3000 ) {
      irChannel = 0;
    if (ppmInput[3] > 3500 ) {
      irChannel = 1;
    }
  }
  // blink the status led based on the IR channel
  byte i = 0;
  while (i <= irChannel) {
    toggleLed ();
    delay(500);
    toggleLed();
    delay(500);
    i++;
  }
  delay(1000);// to separate the channel flashing from the data flashing
}

void loop()
{  
  static byte irData [4];
  static long nextCommandTime;
  static unsigned long throttleWatchdog;
  // ppm data comes in as RETA
  // IR data goes out as RETX ( where x is the unused trim data )
  // This is S107 dependent so a different mechanism will need to be developed to support multiple models
if (ppmInput[3] > 3500)
{ irVideo = 1;
}
else 
{ irVideo = 0;
}

if (ppmInput[3] < 2500)
{ irPicture = 1;
}
else 
{
  irPicture = 0;
}
  irData[0] = ((map(ppmInput[0],1980,4025,255,0)) >> 1) | (irVideo << 7); // rudder, 0-255, reversed
  irData[1] = ((map(ppmInput[1],1980,4025,255,0)) >> 1) | (irPicture << 7); // elevator, 0-255, reversed
  irData[2] = ((map(ppmInput[2],1980,4025,0,255)) >> 1) | (irChannel << 7); // throttle, 0-255, with IR channel as 7th bit
  irData[3] = 63; // this the trim on the original TX.  we just use the rudder trim on the RC TX. hardcoded to mid value

  if (millis() > nextCommandTime)
  {
    nextCommandTime = (millis() + irChannelSpacing[irChannel]);// data rate is based on channel

    // check to see if throttle is above zero
    if (ppmInput[2] > 2000)
    {
      throttleWatchdog = 0;
    }
    else
    {
      throttleWatchdog++;
    }

    if (throttleWatchdog < 6) // throttle is on, or has been recently, send out some data
    {
      sendPreamble();
      byte  i, j;
      for (i = 0; i < sizeof(irData); i++)
      {
        for (j = 0; j < irDataStructure[i]; j++)
        {
          if (irData[i] & _BV((irDataStructure[i] - 1) - j))
          {
            sendOne();
          }
          else
            sendZero();
        }
      }
      sendPostamble(); // this is the postamble bit
      toggleLed ();// flashing status led indicates data has been sent
    }
    else
    {
      // throttle hasnt been on for awhile, leave status led on solid to indicate power
      digitalWrite(LED_OUTPUT_PIN, LOW);  // not flashing. no data going out

    }
  }
}

